/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author      (  )
 * @date       2024
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "VisualizationCylindroid.h"

#include <Inventor/nodes/SoMaterial.h>

bool
armarx::viz::coin::VisualizationCylindroid::update(ElementType const& element)
{
    auto color = element.color;
    constexpr float conv = 1.0f / 255.0f;
    const float r = color.r * conv;
    const float g = color.g * conv;
    const float b = color.b * conv;
    const float a = color.a * conv;

    VirtualRobot::VisualizationNodePtr cylindroid_node;
    {
        // Params.
        SoMaterial* mat = new SoMaterial;
        mat->diffuseColor.setValue(r, g, b);
        mat->ambientColor.setValue(r, g, b);
        mat->transparency.setValue(1. - a);

        SoSeparator* res = new SoSeparator();
        res->ref();
        SoUnits* u = new SoUnits();
        u->units = SoUnits::MILLIMETERS;
        res->addChild(u);
        res->addChild(VirtualRobot::CoinVisualizationFactory::CreateCylindroid(
            element.axisLengths.e0, element.axisLengths.e1, element.height, mat));

        cylindroid_node.reset(new VirtualRobot::CoinVisualizationNode(res));
        res->unref();
    }

    SoNode* cylindroid =
        dynamic_cast<VirtualRobot::CoinVisualizationNode&>(*cylindroid_node).getCoinVisualization();

    node->removeAllChildren();
    node->addChild(cylindroid);

    return true;
}
