/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    robdekon
 * @author     Christoph Pohl ( christoph dot pohl at kit dot edu )
 * @date       06.09.22
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <ArmarXCore/core/Component.h>

#include <RobotAPI/components/AronComponentConfigExample/aron/ComponentConfig.aron.generated.h>
#include <RobotAPI/libraries/aron_component_config/ComponentPlugin.h>

namespace armarx
{

    class AronComponentConfigExamplePropertyDefinitions :
        public armarx::ComponentPropertyDefinitions
    {
    public:
        AronComponentConfigExamplePropertyDefinitions(std::string prefix);
    };

    class AronComponentConfigExample : virtual public armarx::Component
    {
    public:
        std::string getDefaultName() const override;

        /// @see armarx::ManagedIceObject::onInitComponent()
        void onInitComponent() override;

        /// @see armarx::ManagedIceObject::onConnectComponent()
        void onConnectComponent() override;

        /// @see armarx::ManagedIceObject::onDisconnectComponent()
        void onDisconnectComponent() override;

        /// @see armarx::ManagedIceObject::onExitComponent()
        void onExitComponent() override;

        /// @see PropertyUser::createPropertyDefinitions()
        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;

        AronComponentConfigExample();

    private:
        SimplePeriodicTask<>::pointer_type periodicTask;
        std::experimental::observer_ptr<plugins::RemoteGuiComponentPlugin> remote_gui_plugin_{
            nullptr};
        std::experimental::observer_ptr<
            plugins::AronComponentConfigPlugin<armarx::component_config::aron::TestConfig>>
            aron_component_config_plugin_{nullptr};
        std::mutex write_mutex;
    };
} // namespace armarx