/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::RobotAndObjectToIndex
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <vector>

#include <ArmarXCore/core/time/Frequency.h>

#include <RobotAPI/libraries/ArmarXObjects/ObjectPoseClient.h>
#include <RobotAPI/libraries/armem/client/Writer.h>
#include <RobotAPI/libraries/armem/core/MemoryID.h>

namespace armarx::armem::objects
{

    class ObjectInstanceToIndex
    {
    public:
        void fetchAndCommitObjectInstances(
            const std::vector<armem::MemoryID>& updatedObjectInstanceSnapshotIDs);

    private:
        std::vector<const objpose::ObjectPose*>
        filterObjectPoses(const objpose::ObjectPoseSeq& objectPoses,
                          const std::vector<armem::MemoryID>& updatedObjectInstanceSnapshotIDs);

    public:
        objpose::ObjectPoseClient objectPoseClient;
        armem::client::Writer indexSpatialMemoryWriter;

        armem::MemoryID indexSpatialProviderSegmentID;
        armem::MemoryID indexNamedProviderSegmentID;

        struct Parameters
        {
            armarx::Frequency maxFrequency = armarx::Frequency::HertzDouble(60);
        };

        Parameters params;

        struct State
        {
            std::map<armarx::ObjectID, armarx::DateTime> latestUpdateDateTimes;
        };

        State state;
    };

} // namespace armarx::armem::objects
