/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "LaserScansMemory.h"

#include <SimoxUtility/algorithm/string.h>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include "RobotAPI/libraries/armem/server/MemoryToIceAdapter.h"
#include "RobotAPI/libraries/armem_laser_scans/constants.h"
#include <RobotAPI/libraries/armem/core/error.h>
#include <RobotAPI/libraries/armem/server/MemoryRemoteGui.h>
#include <RobotAPI/libraries/armem_laser_scans/aron/LaserScan.aron.generated.h>

namespace armarx::armem::server::laser_scans
{
    LaserScansMemory::LaserScansMemory()
    {
        addPlugin(virtualRobotReaderPlugin);
        addPlugin(debugObserver);
    }

    armarx::PropertyDefinitionsPtr
    LaserScansMemory::createPropertyDefinitions()
    {
        armarx::PropertyDefinitionsPtr defs =
            new ComponentPropertyDefinitions(getConfigIdentifier());

        workingMemory().name() = "LaserScans";

        {
            const std::string prefix = "";
            commonVisu.defineProperties(defs, prefix + "visu.");
        }

        return defs;
    }

    std::string
    LaserScansMemory::getDefaultName() const
    {
        return "LaserScansMemory";
    }

    void
    LaserScansMemory::onInitComponent()
    {
        const auto& coreSegment = workingMemory().addCoreSegment(
            "LaserScans", armarx::armem::laser_scans::arondto::LaserScanStamped::ToAronType());

        commonVisu.init(&coreSegment, &virtualRobotReaderPlugin->get());
    }

    void
    LaserScansMemory::onConnectComponent()
    {
        commonVisu.connect(getArvizClient(), debugObserver->getDebugObserver());
    }

    void
    LaserScansMemory::onDisconnectComponent()
    {
    }

    void
    LaserScansMemory::onExitComponent()
    {
    }

    armem::data::AddSegmentsResult
    LaserScansMemory::addSegments(const armem::data::AddSegmentsInput& input, const Ice::Current&)
    {
        // Allowing adding core segments.
        armem::data::AddSegmentsResult result =
            ReadWritePluginUser::addSegments(input, addCoreSegmentOnUsage);
        return result;
    }

} // namespace armarx::armem::server::laser_scans
