/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::NJointHolonomicPlatformVelocityControllerInterface
 * @author     Tobias Gröger ( tobias dot groeger at student dot kit dot edu )
 * @date       2024
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include "NJointControllerWithTripleBuffer.h"

namespace armarx
{

    TYPEDEF_PTRS_HANDLE(NJointHolonomicPlatformVelocityControllerControlData);

    class NJointHolonomicPlatformVelocityControllerControlData
    {
    public:
        float velocityX = 0;
        float velocityY = 0;
        float velocityRotation = 0;
        IceUtil::Time commandTimestamp;
    };

    TYPEDEF_PTRS_HANDLE(NJointHolonomicPlatformVelocityControllerInterface);

    class NJointHolonomicPlatformVelocityControllerInterface :
        virtual public NJointControllerWithTripleBuffer<
            NJointHolonomicPlatformVelocityControllerControlData>
    {
    public:
        NJointHolonomicPlatformVelocityControllerInterface();


        void setVelocites(float velocityX, float velocityY, float velocityRotation);

        IceUtil::Time getMaxCommandDelay() const;
        void setMaxCommandDelay(const IceUtil::Time& value);

    protected:
        IceUtil::Time maxCommandDelay;
    };
} // namespace armarx
