/*
 * IMUDeducedReckoning.h
 *
 *  Created on: Mar 16, 2014
 *      Author: Dr.-Ing. David Israel González Aguirre
 *      Mail:   david.gonzalez@kit.edu
 */

#pragma once

#include "IIMUEventDispatcher.h"
#include "IMUDevice.h"
#include "IMUState.h"

namespace IMU
{
    class CIMUDeducedReckoning : public IIMUEventDispatcher
    {
    public:
        CIMUDeducedReckoning(const bool IsVerbose);
        ~CIMUDeducedReckoning() override;

        inline const float*
        GetOrientationQuaternion() const
        {
            return m_OrientationQuaternion;
        }

        inline const float*
        GetMagneticRotation() const
        {
            return m_MagneticRotation;
        }

        inline const float*
        GetGyroscopeRotation() const
        {
            return m_GyroscopeRotation;
        }

        inline const float*
        GetAccelaration() const
        {
            return m_Accelaration;
        }

    protected:
        void OnIMUEvent(const CIMUEvent& Event) override;

        void OnIMUStart(const timeval& TimeStamp, const CIMUDevice* pIMUDevice);
        void OnIMUStop(const timeval& TimeStamp, const CIMUDevice* pIMUDevice);
        void OnIMUFusedCycle(const timeval& TimeStamp, const CIMUDevice* pIMUDevice);
        void OnIMUCycle(const timeval& TimeStamp, const CIMUDevice* pIMUDevice);
        void OnIMUIntegratedState(const timeval& TimeStamp, const CIMUDevice* pIMUDevice);
        void OnIMUCustomEvent(const CIMUEvent& CustomEvent);

        const bool m_IsVerbose;
        const float m_G;

        float m_OrientationQuaternion[4];
        float m_MagneticRotation[3];
        float m_GyroscopeRotation[3];
        float m_Accelaration[3];
    };
} // namespace IMU
