#pragma once

#include <cmath>
#include <vector>

#include <QLineEdit>
#include <QObject>
#include <QVBoxLayout>

#include <SimoxUtility/algorithm/string/string_conversion.h>
#include <SimoxUtility/error/SimoxError.h>

#include <RobotAPI/interface/aron/Aron.h>

#include "CustomWidget.h"

namespace armarx
{
    // Custom Widget which represents a Quaternion - This class does not normalize the inputs
    // It can parse its data to NDArray and also handles user edit signals itself. (And also the highlighting if errors occur)
    class QuaternionWidget : public CustomWidget
    {
        Q_OBJECT
    public:
        QuaternionWidget(aron::type::quaternion::ElementType type, QTreeWidgetItem* currentItem);

        static QuaternionWidget* DynamicCast(QWidget*);
        static QuaternionWidget* DynamicCastAndCheck(QWidget*);

        enum struct QuaternionComponents
        {
            X = 0,
            Y,
            Z,
            W
        };
        void setText(QuaternionComponents col, const std::string& str);
        std::string getText(QuaternionComponents col);
        bool hasParseErrors();
        std::vector<unsigned char> parseAllToNDArray();

    private:
        std::vector<QLineEdit*> xyzw_components;
        aron::type::quaternion::ElementType element_type;
        bool parseErrors = false;

        void highlightUnparsableEntries();
        template <typename T>
        std::pair<bool, T> parseQuatVals(QuaternionComponents comp);

    private slots:
        void quaternionElementChanged();
    };

    template <typename T>
    std::pair<bool, T>
    QuaternionWidget::parseQuatVals(QuaternionWidget::QuaternionComponents comp)
    {
        // size_t data_size = element_type == aron::type::quaternion::ElementType::FLOAT32 ? 4 : 8;
        try
        {
            T val = simox::alg::to_<T>(xyzw_components.at((size_t)comp)->text().toStdString());
            return {true, val};
        }
        catch (const simox::error::SimoxError& err)
        {
            return {false, NAN};
        }
    }


} // namespace armarx
