/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Simon Ottenhaus ( simon dot ottenhaus at kit dot edu )
 * @date       2018
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */


#pragma once

#include <ArmarXCore/interface/core/BasicTypes.ice>
#include <ArmarXCore/interface/core/time.ice>

module armarx
{

    enum RobotHealthState
    {
        HealthOK,
        HealthWarning,
        HealthError
    };

    struct RobotHealthHeartbeatArgs
    {
        string identifier;
        // bool requiredByDefault;
        string description;
        Ice::StringSeq tags;

        armarx::core::time::dto::Duration maximumCycleTimeWarning;
        armarx::core::time::dto::Duration maximumCycleTimeError;
    };

    interface RobotHealthInterface
    {
        void signUp(RobotHealthHeartbeatArgs args);
        void unregister(string identifier);

        void addRequiredTags(string requesterIdentifier, Ice::StringSeq tags);
        void removeRequiredTags(string requesterIdentifier, Ice::StringSeq tags);
        void resetRequiredTags();

        void heartbeat(string identifier, armarx::core::time::dto::DateTime referenceTime);

        string getTopicName();
    };

    // Used by the RobotHealth to send an overall status update ehich e.g. the emergency stop listens to.
    interface AggregatedRobotHealthInterface
    {
        void aggregatedHeartbeat(RobotHealthState overallHealthState);
    };

    struct RobotHealthInfoEntry
    {
        string identifier;
        RobotHealthState state;

        // string message;
        armarx::core::time::dto::Duration minDelta;
        armarx::core::time::dto::Duration maxDelta;

        armarx::core::time::dto::DateTime lastReferenceTimestamp;

        //< Time delta to now() when arrived at heart beat component
        armarx::core::time::dto::Duration timeSinceLastArrival;

        //< Time delta to reference timestamp sent by component
        armarx::core::time::dto::Duration timeSinceLastUpdateReference;

        armarx::core::time::dto::Duration timeSyncDelayAndIce;

        bool required; //
        bool enabled;

        armarx::core::time::dto::Duration maximumCycleTimeWarning;
        armarx::core::time::dto::Duration maximumCycleTimeError;

        Ice::StringSeq tags;
    };

    dictionary<string, RobotHealthInfoEntry> RobotHealthEntries;

    struct RobotHealthInfo
    {
        Ice::StringSeq activeTags;
        RobotHealthEntries entries;
    };

    interface RobotHealthComponentInterface extends RobotHealthInterface
    {
        RobotHealthInfo getSummary();
    };
};
