/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI
 * @author     Martin Do ( martin dot do at kit dot edu )
 * @date       2015
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <ArmarXCore/interface/core/UserException.ice>
#include <ArmarXCore/interface/observers/Timestamp.ice>

#include <RobotAPI/interface/core/PoseBase.ice>
#include <RobotAPI/interface/units/UnitInterface.ice>

module armarx
{
    /**
     * Will be thrown, if a Arduino error occurs.
     */
    exception ArduinoException extends UserException
    {
        int errorCode;
    };


    interface OrientedTactileSensorUnitInterface extends armarx::SensorActorUnitInterface{
        //bool loadCalibration(int accel_offset_x, int accel_offset_y, int accel_offset_z, int gyro_offset_x, int gyro_offset_y, int gyro_offset_z, int mag_offset_x, int mag_offset_y, int mag_offset_z, int accel_radius, int mag_radius );
    };

    interface OrientedTactileSensorUnitListener
    {
        void reportSensorValues(int id,
                                float pressure,
                                float qw,
                                float qx,
                                float qy,
                                float qz,
                                float pressureRate,
                                float rotationRate,
                                float accelerationRate,
                                float accelx,
                                float accely,
                                float accelz,
                                TimestampBase timestamp);
    };


    interface OrientedTactileSensorUnitObserverInterface extends ObserverInterface,
        OrientedTactileSensorUnitListener{

        };
};
