#include "ObjectPoseProviderPlugin.h"

#include <ArmarXCore/core/Component.h>

namespace armarx::plugins
{

    void
    ObjectPoseProviderPlugin::postCreatePropertyDefinitions(PropertyDefinitionsPtr& properties)
    {
        if (!properties->hasDefinition(makePropertyName(PROPERTY_NAME)))
        {
            properties->defineOptionalProperty<std::string>(
                makePropertyName(PROPERTY_NAME), "ObjectMemory", "Name of the object memory.");
        }
    }

    void
    ObjectPoseProviderPlugin::preOnInitComponent()
    {
        parent<Component>().usingProxyFromProperty(makePropertyName(PROPERTY_NAME));
    }

    void
    ObjectPoseProviderPlugin::preOnConnectComponent()
    {
        parent<ObjectPoseProviderPluginUser>().objectPoseTopic =
            parent<Component>().getProxyFromProperty<objpose::ObjectPoseStorageInterfacePrx>(
                makePropertyName(PROPERTY_NAME));
    }

    void
    ObjectPoseProviderPlugin::postOnConnectComponent()
    {
        auto& parent = this->parent<ObjectPoseProviderPluginUser>();
        if (parent.objectPoseTopic)
        {
            objpose::ProviderInfo info = parent.getProviderInfo();
            if (!info.proxy)
            {
                info.proxy =
                    ::armarx::objpose::ObjectPoseProviderPrx::checkedCast(parent.getProxy());
            }

            ARMARX_INFO << "Reporting provider `" << parent.getName() << "` availability.";
            parent.objectPoseTopic->reportProviderAvailable(parent.getName(), info);
        }
        else
        {
            ARMARX_ERROR << "Object pose topic not ready.";
        }
    }

} // namespace armarx::plugins

namespace armarx
{

    ObjectPoseProviderPluginUser::ObjectPoseProviderPluginUser()
    {
        addPlugin(plugin);
    }

    objpose::provider::RequestObjectsOutput
    ObjectPoseProviderPluginUser::requestObjects(
        const objpose::provider::RequestObjectsInput& input,
        const Ice::Current&)
    {
        // ARMARX_INFO << "Requested " << input.objectIDs.size() << " objects for " << input.relativeTimeoutMS << " ms.";
        objpose::provider::RequestObjectsOutput output;
        for (const auto& id : input.objectIDs)
        {
            output.results[id].success = false;
        }
        return output;
    }


} // namespace armarx
