#include "AffordanceLoader.h"

#include <ArmarXCore/core/eigen/ice_conversions.h>
#include <ArmarXCore/core/logging/Logging.h>

namespace armarx::priorknowledge::util
{
    std::vector<StaticAffordance>
    AffordanceLoader::LoadAffordances(const std::string& source, const nlohmann::json& js)
    {
        std::vector<StaticAffordance> ret;
        if (not js.contains("affordances"))
        {
            ARMARX_WARNING << "The affordances file has the wrong structure. Missing key "
                              "'affordances'.";
            return ret;
        }

        for (const auto& affordance : js["affordances"].get<std::vector<std::string>>())
        {
            StaticAffordance a{{source, affordance}};
            ret.push_back(a);
        }
        return ret;
    }

    std::vector<LocationAffordance>
    AffordanceLoader::LoadLocationAffordances(const std::string& source, const nlohmann::json& js)
    {
        std::vector<LocationAffordance> ret;
        if (not js.contains("location_affordances"))
        {
            ARMARX_WARNING << "The affordances file has the wrong structure. Missing key "
                              "'location_affordances'.";
            return ret;
        }

        for (const auto& [locationName, affordances] :
             js["location_affordances"].get<std::map<std::string, std::vector<std::string>>>())
        {
            for (const auto& affordance : affordances)
            {
                LocationAffordance a{{source, affordance}, locationName};
                ret.push_back(a);
            }
        }
        return ret;
    }
} // namespace armarx::priorknowledge::util
