#pragma once

#include <regex>
#include <unordered_map>

#include <RobotAPI/components/ArViz/Client/Client.h>
#include <RobotAPI/components/ArViz/Client/ScopedClient.h>

#include "datatypes/Location.h"

namespace armarx::priorknowledge::util::location
{

    class Visu
    {
    public:
        struct FramedLocationData
        {
            Eigen::Matrix4f globalPose;
            std::optional<Eigen::Vector3f> extents;
        };

        Visu(viz::Client& arviz) : arviz(arviz)

        {
        }

        ~Visu() = default;

        viz::Layer
        locationsToLayer(const std::string& layerName,
                         const std::map<std::string, FramedLocationData>& locationData) const;

        viz::Layer framedLocationsToLayer(
            const std::string& layerName,
            const std::map<std::string, Eigen::Matrix4f>& locationGlobalPoses) const;

        viz::Layer framedBoxedLocationsToLayer(
            const std::string& layerName,
            const std::map<std::string, std::pair<Eigen::Matrix4f, Eigen::Vector3f>>&
                locationGlobalPosesAndExtends) const;

    protected:
        void addFramedLocationToLayer(viz::Layer&,
                                      const std::string& id,
                                      const Eigen::Matrix4f& locationGlobalPose) const;

        void addFramedBoxedLocationToLayer(viz::Layer&,
                                           const std::string& id,
                                           const Eigen::Matrix4f& locationGlobalPose,
                                           const Eigen::Vector3f& extends) const;

    public:
        struct Settings
        {
            // FramedLocation
            viz::Color framedLocationArrowColor = viz::Color::green();

            // FramedBoxedLocation
            viz::Color framedBoxedLocationColor = viz::Color(0, 150, 130, 40);

        } settings;

    protected:
        viz::Client& arviz;

    private:
        const std::regex pattern = std::regex(R"(^[^:]+\/([^:]+?):[^:]+?$)");
        mutable std::set<std::string> unique_affordances;
    };
} // namespace armarx::priorknowledge::util::location
