#include "SimpleWriterBase.h"

#include <RobotAPI/libraries/armem/client/MemoryNameSystem.h>
#include <RobotAPI/libraries/armem/core/error.h>

namespace armarx::armem::client::util
{
    SimpleWriterBase::SimpleWriterBase()
    {
    }

    void
    SimpleWriterBase::registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& def)
    {
        ARMARX_DEBUG << "Writer: registerPropertyDefinitions";

        const std::string prefix = propertyPrefix();

        if (not props.has_value())
        {
            props = defaultProperties();
        }

        def->optional(props->memoryName, prefix + "Memory");
        def->optional(props->coreSegmentName, prefix + "CoreSegment");

        // TODO(fabian.reister): this might also be part of the subclass
        //  if the provider name has to be derived from e.g. the component name
        def->optional(props->providerName, prefix + "Provider", "Name of this provider");
    }

    void
    SimpleWriterBase::connect(armarx::armem::client::MemoryNameSystem& mns)
    {
        // Wait for the memory to become available and add it as dependency.
        ARMARX_IMPORTANT << "SimpleWriterBase: Waiting for memory '" << properties().memoryName
                         << "' ...";
        try
        {
            memoryWriterClient = mns.useWriter(MemoryID().withMemoryName(properties().memoryName));
            ARMARX_IMPORTANT << "SimpleWriterBase: Connected to memory '" << properties().memoryName
                             << "'";
        }
        catch (const armem::error::CouldNotResolveMemoryServer& e)
        {
            ARMARX_ERROR << e.what();
            return;
        }
    }

    armem::client::Writer&
    SimpleWriterBase::memoryWriter()
    {
        return memoryWriterClient;
    }

    const SimpleWriterBase::Properties&
    SimpleWriterBase::properties() const
    {
        if (not props.has_value())
        {
            const_cast<std::optional<Properties>&>(props) = defaultProperties();
        }
        return props.value();
    }

    void
    SimpleWriterBase::setProperties(const Properties& p)
    {
        props = p;
    }

} // namespace armarx::armem::client::util
