#pragma once

// STD/STL
#include <memory>

// Simox
#include <SimoxUtility/json.h>

// ArmarX
#include "RobotAPI/libraries/aron/similarity/data/image/NDArraySimilarity.h"
#include <RobotAPI/libraries/armem/core/MemoryID.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>

namespace armarx::armem::server::ltm::processor
{
    class MemoryFilter
    {
    public:
        MemoryFilter() = default;
        virtual ~MemoryFilter() = default;

        virtual bool accept(const armem::wm::Memory& e) = 0;
        virtual void configure(const nlohmann::json& json);
    };

    class SnapshotFilter
    {
    public:
        SnapshotFilter() = default;
        virtual ~SnapshotFilter() = default;

        virtual bool accept(const armem::wm::EntitySnapshot& e, bool simulatedVersion = false) = 0;
        virtual void configure(const nlohmann::json& json);

        struct FilterStatistics
        {
            double accepted = 0;
            double rejected = 0;
            std::chrono::duration<double> additional_time = std::chrono::duration<double>::zero();
            std::string additional_info = "";
            aron::similarity::NDArraySimilarity::Type similarity_type;
            std::chrono::high_resolution_clock::time_point start_time;
            std::chrono::high_resolution_clock::time_point end_time;
            int number_of_compared_objects = 2;
            std::string importance_type = "";
        } stats;

        virtual FilterStatistics getFilterStatistics();
        virtual std::string getName();

        /**
         * @brief resetStatisticsForNewEpisode resets the parts of the statistics that are unique
         * to one recording cycle (episode)
         */
        void resetStatisticsForNewEpisode();
    };
} // namespace armarx::armem::server::ltm::processor
