#pragma once

#include <map>
#include <vector>

// Base Class
#include "../Filter.h"

// Aron
#include <chrono>

#include "RobotAPI/libraries/aron/similarity/data/image/FloatSimilarity.h"
#include "RobotAPI/libraries/aron/similarity/data/image/NDArraySimilarity.h"
#include <RobotAPI/libraries/aron/core/data/variant/container/Dict.h>

namespace armarx::armem::server::ltm::processor::filter
{
    class SnapshotSimilarityFilter : public SnapshotFilter
    {
    public:
        static const constexpr char* NAME = "SnapshotSimilarityFilter";
        static const constexpr char* PARAM_THRESHOLD = "Threshold";
        static const constexpr char* PARAM_SIM_MEASURE = "SimilarityMeasure";
        static const constexpr char* PARAM_MAX_OBJECTS = "NumberOfObjectsToCompare";

        SnapshotSimilarityFilter() = default;

        virtual bool accept(const armem::wm::EntitySnapshot& e, bool simulatedVersion) override;
        void configure(const nlohmann::json& json) override;

        FilterStatistics getFilterStatistics() override;
        std::string getName() override;

    private:
        //std::map<MemoryID, std::vector<aron::data::DictPtr>> dataLastCommit;
        std::deque<std::vector<armarx::aron::data::NDArrayPtr>> images;
        std::deque<std::vector<armarx::aron::data::FloatPtr>> floats;
        std::map<MemoryID, long> timestampLastCommitInMs;
        std::double_t threshold;
        FilterStatistics stats;
        std::size_t max_images = 2;
        aron::similarity::NDArraySimilarity::Type similarity_type;
        aron::similarity::FloatSimilarity::Type float_similarity_type;
    };
} // namespace armarx::armem::server::ltm::processor::filter
