#pragma once

#include <mutex>

#include <RobotAPI/libraries/armem/client/query/Builder.h>
#include <RobotAPI/libraries/armem/core/query/DataMode.h>

#include "SnapshotSelectorWidget.h"

class QWidget;
class QTabWidget;
class QPushButton;
class QGroupBox;
class QLineEdit;
class QString;

namespace armarx::armem::gui
{

    class QueryWidget : public QWidget
    {
        Q_OBJECT
        using This = QueryWidget;

    public:
        enum class ActiveMemoryState
        {
            FoundAndChecked,
            FoundAndNotChecked,
            NotFound
        };

        QueryWidget();

        armem::query::DataMode dataMode() const;

        bool dropRemovedMemories() const;
        bool dropDisabledMemories() const;

        int queryLinkRecursionDepth() const;

        std::map<std::string, ActiveMemoryState> getAvailableMemoryStates() const;

        std::vector<std::string> getEnabledMemories() const;

        void update(const std::vector<std::string>& memoryNames);


    public slots:

    signals:
        void storeInLTM();
        void startRecording();
        void stopRecording();
        //void confirmExportPath(std::string path);

        // ToDo:
        // void queryChanged(armem::query::data::Input query);


    private slots:
        void setRecursionDepthSpinnerEnabled(int state);

        void deSelectMemoryServers();

    signals:


    private:
        QCheckBox* _dataCheckBox;

        QCheckBox* _dropRemovedCheckBox;
        QCheckBox* _dropDisabledCheckBox;

        QLabel* _recursionDepthLabel;
        QSpinBox* _recursionDepthSpinner;

        QLabel* _exportPathLabel;
        QLineEdit* _exportPathLineEdit;
        QPushButton* _exportPathConfirmButton;

        QLabel* _exportNameLabel;
        QLineEdit* _exportNameLineEdit;
        QPushButton* _exportNameConfirmButton;

        QPushButton* _storeInLTMButton;
        QPushButton* _startLTMRecordingButton;
        QPushButton* _stopLTMRecordingButton;

        QGroupBox* _additionalSettingsGroupBox;
        QGroupBox* _availableMemoriesGroupBox;

        QPushButton* _de_selectMemoryServers;
        bool allMemoryServersSelected = false;
        QString selectText = "Select all Memory Servers";
        QString deselectText = "Deselect all Memory Servers";

        mutable std::mutex enabledMemoriesMutex;
    };

} // namespace armarx::armem::gui
