/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <optional>

#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>

#include <RobotAPI/interface/objectpose/FamiliarObjectPoseStorageInterface.h>
#include <RobotAPI/libraries/armem/client/MemoryNameSystem.h>
#include <RobotAPI/libraries/armem/client/Reader.h>
#include <RobotAPI/libraries/armem_objects/aron/FamiliarObjectInstance.aron.generated.h>
#include <RobotAPI/libraries/armem_objects/aron/ObjectInstance.aron.generated.h>
#include <RobotAPI/libraries/armem_objects/types.h>

// The object pose provider. As long as the provider is not connected to armem we need the second proxy
#include <RobotAPI/interface/objectpose/ObjectPoseProvider.h>
#include <RobotAPI/interface/objectpose/ObjectPoseStorageInterface.h>

namespace armarx::armem::obj::familiar_object_instance
{
    class Reader
    {
    public:
        // using StoragePrxType = objpose::FamiliarObjectPoseStorageInterfacePrx;

        struct Properties
        {
            std::string memoryName = "Object";
        };

        Reader() = default;
        virtual ~Reader() = default;

        void registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& def);
        void connect(armem::client::MemoryNameSystem& memoryNameSystem);

        // query existing instance from the memory. Requires an instance index to be set.
        std::optional<std::map<std::string, arondto::FamiliarObjectInstance>>
        queryLatestFamiliarObjectInstance(
            const ObjectID& instanceId,
            const std::optional<std::string>& providerName = std::nullopt) const;

        // query the latest instances. Ignores the instance index.
        std::map<std::string, std::vector<arondto::FamiliarObjectInstance>>
        queryAllLatestFamiliarObjectInstances(
            const std::optional<std::string>& providerName = std::nullopt) const;

        // query all latest object instances.
        std::map<std::string, std::vector<arondto::FamiliarObjectInstance>>
        queryLatestFamiliarObjectInstancesFromClass(
            const ObjectID& classId,
            const std::optional<std::string>& providerName = std::nullopt) const;

        Properties
        getProperties()
        {
            return this->p;
        }

        // StoragePrxType
        // getFamiliarObjectPoseStorage() const
        // {
        //     return famObjPosStorage;
        // }

    private:
        Properties p;

        const std::string propertyPrefix = "mem.obj.familiar_object_instance.";

        armem::client::Reader memoryReader;


        // StoragePrxType famObjPosStorage;
    };


} // namespace armarx::armem::obj::familiar_object_instance
