/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>

#include <RobotAPI/libraries/armem/client/MemoryNameSystem.h>
#include <RobotAPI/libraries/armem/client/Writer.h>
#include <RobotAPI/libraries/armem_objects/aron/FamiliarObjectInstance.aron.generated.h>
#include <RobotAPI/libraries/armem_objects/types.h>

namespace armarx::armem::obj::familiar_object_instance
{
    class Writer
    {
    public:
        Writer();
        virtual ~Writer() = default;

        void registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& def);
        void connect(armem::client::MemoryNameSystem& memoryNameSystem);

        bool commitObject(const armem::arondto::FamiliarObjectInstance& inst,
                          const std::string& provider,
                          const armem::Time&);

    private:
        struct Properties
        {
            std::string memoryName = "Object";
            std::string coreSegmentName = "FamiliarObjectInstance";
        } properties;

        const std::string propertyPrefix = "mem.obj.familiar_object_instance.";

        armem::client::Writer memoryWriter;
    };


} // namespace armarx::armem::obj::familiar_object_instance
