/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <mutex>
#include <optional>

#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>

#include <RobotAPI/libraries/armem/client/MemoryNameSystem.h>
#include <RobotAPI/libraries/armem/client/Reader.h>
#include <RobotAPI/libraries/armem_objects/aron/ObjectInstance.aron.generated.h>
#include <RobotAPI/libraries/armem_objects/types.h>

// The object pose provider. As long as the provider is not connected to armem we need the second proxy
#include <RobotAPI/interface/armem/server/MemoryInterface.h>
#include <RobotAPI/interface/objectpose/ObjectPoseProvider.h>
#include <RobotAPI/interface/objectpose/ObjectPoseStorageInterface.h>

namespace armarx::armem::obj::instance
{
    class Reader
    {
    public:
        struct Properties
        {
            std::string memoryName = "Object";
        };

        Reader() = default;
        virtual ~Reader() = default;

        void registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& def);
        void connect(armem::client::MemoryNameSystem& memoryNameSystem);

        // localization stuff. Requires an instance index to be set.
        std::map<std::string, bool> requestLocalization(const ObjectID& instanceId,
                                                        const armarx::core::time::Duration& until);
        bool requestLocalization(const ObjectID& instanceId,
                                 const std::string& provider,
                                 const armarx::core::time::Duration& until);

        // query existing instance from the memory. Requires an instance index to be set.
        std::optional<objpose::ObjectPose> queryLatestObjectInstance(const ObjectID& instanceId);

        // query the latest instances. Ignores the instance index.
        std::map<std::string, objpose::ObjectPose>
        queryLatestObjectInstances(const ObjectID& classId);

        // query all latest object instances.
        std::map<std::string, objpose::ObjectPose> queryLatestObjectInstances();

        Properties
        getProperties()
        {
            return this->properties;
        }

        objpose::ObjectPoseStorageInterfacePrx
        getObjectPoseStorage() const
        {
            // TODO: see problem in source file. This performs a ice_isA every time this method is called
            return objpose::ObjectPoseStorageInterfacePrx::checkedCast(this->readingPrx);
        }

    private:
        Properties properties;

        const std::string propertyPrefix = "mem.obj.instance.";

        armarx::armem::server::ReadingMemoryInterfacePrx readingPrx;
    };


} // namespace armarx::armem::obj::instance
