/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::armem_objects::Adapter
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2024
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once


#include <experimental/memory>
#include <mutex>

#include <VirtualRobot/VirtualRobot.h>

#include <ArmarXGui/libraries/RemoteGui/Client/Widgets.h>

#include "RobotAPI/libraries/armem_robot_state/client/common/VirtualRobotReader.h"
#include <RobotAPI/components/ArViz/Client/Client.h>
#include <RobotAPI/interface/armem/server/ObjectMemoryInterface.h>
#include <RobotAPI/interface/core/RobotState.h>
#include <RobotAPI/libraries/armem/server/MemoryToIceAdapter.h>
#include <RobotAPI/libraries/armem_objects/server/familiar_object_instance/Decay.h>
#include <RobotAPI/libraries/armem_objects/server/familiar_object_instance/Segment.h>
#include <RobotAPI/libraries/armem_objects/server/familiar_object_instance/Visu.h>


#define ICE_CURRENT_ARG const Ice::Current& = Ice::emptyCurrent

namespace armarx::armem::server::obj::familiar_object_instance
{

    /**
     * @brief Helps implementing the `armarx::armem::server::ObjectInstanceSegmentInterface`.
     */
    class SegmentAdapter :
        virtual public armarx::Logging,
        virtual public armarx::armem::server::FamiliarObjectInstanceSegmentInterface
    {
    public:
        SegmentAdapter(MemoryToIceAdapter& iceMemory);

        std::string getName() const;
        void defineProperties(armarx::PropertyDefinitionsPtr defs, const std::string& prefix = "");

        void init();
        void
        connect(std::experimental::observer_ptr<robot_state::VirtualRobotReader> virtualRobotReader,
                // KinematicUnitObserverInterfacePrx kinematicUnitObserver,
                viz::Client arviz,
                DebugObserverInterfacePrx debugObserver);


        // ObjectPoseTopic interface
    public:
        // void reportProviderAvailable(const std::string& providerName,
        //                              const objpose::ProviderInfo& info,
        //                              ICE_CURRENT_ARG) override;
        void reportFamiliarObjectPoses(const std::string& providerName,
                                       const ::armarx::objpose::ProvidedFamiliarObjectPoseSeq& data,
                                       ICE_CURRENT_ARG) override;

        // ObjectInstanceSegmentInterface interface


    private:
        void updateProviderInfo(const std::string& providerName, const objpose::ProviderInfo& info);

        void updateObjectPoses(const std::string& providerName,
                               const objpose::data::ProvidedObjectPoseSeq& providedPoses);
        void handleProviderUpdate(const std::string& providerName);


        // Visualization

        void visualizeRun();


    public:
        static const std::string linearPredictionEngineID;
        static const std::vector<PredictionEngine> predictionEngines;


    private:
        viz::Client arviz;
        DebugObserverInterfacePrx debugObserver;

        familiar_object_instance::Segment segment;


        familiar_object_instance::Visu visu;
        std::mutex visuMutex;
    };

} // namespace armarx::armem::server::obj::familiar_object_instance

#undef ICE_CURRENT_ARG
