#pragma once

#include <map>
#include <string>

#include <Eigen/Core>

#include <RobotAPI/libraries/armem_robot_state/aron/Exteroception.aron.generated.h>

#include "ConverterInterface.h"

namespace armarx::armem::server::robot_state::exteroception
{
    struct ConverterValue;

    // class ConverterTools;

    namespace detail
    {
        struct Properties
        {
            std::size_t tofSize = 64; // 4x4 grid
        };
    } // namespace detail

    class ArmarDEConverter : public ConverterInterface
    {
    public:
        using Properties = detail::Properties;

        ArmarDEConverter(const Properties& properties = Properties());
        ~ArmarDEConverter() override;


        aron::data::DictPtr
        convert(const RobotUnitDataStreaming::TimeStep& data,
                const RobotUnitDataStreaming::DataStreamingDescription& description) override;


    protected:
        void process(arondto::Exteroception& dtoExteroception,
                     const std::string& entryName,
                     const ConverterValue& value);


    private:
        void
        processToFDepthEntry(std::map<std::string, armarx::armem::exteroception::arondto::ToF>& fts,
                             const std::vector<std::string>& split,
                             const ConverterValue& value);

        void processToFDepthEntry(armarx::armem::exteroception::arondto::ToF& tof,
                                  const std::vector<std::string>& split,
                                  const ConverterValue& value);

        void
        processToFSigmaEntry(std::map<std::string, armarx::armem::exteroception::arondto::ToF>& fts,
                             const std::vector<std::string>& split,
                             const ConverterValue& value);

        void processToFSigmaEntry(armarx::armem::exteroception::arondto::ToF& tof,
                                  const std::vector<std::string>& split,
                                  const ConverterValue& value);


        void processToFTargetDetectedEntry(
            std::map<std::string, armarx::armem::exteroception::arondto::ToF>& fts,
            const std::vector<std::string>& split,
            const ConverterValue& value);

        void processToFTargetDetectedEntry(armarx::armem::exteroception::arondto::ToF& tof,
                                           const std::vector<std::string>& split,
                                           const ConverterValue& value);


        // std::unique_ptr<ConverterTools> tools;

        Properties properties_;
    };
} // namespace armarx::armem::server::robot_state::exteroception
