/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "Exception.h"

armarx::aron::error::AronException::AronException(const std::string& prettymethod,
                                                  const std::string& reason) :
    LocalException(prettymethod + ": " + reason + ".")
{
}

armarx::aron::error::AronException::AronException(const std::string& prettymethod,
                                                  const std::string& reason,
                                                  const Path& path) :
    LocalException(prettymethod + ": " + reason + ". The path was: " + path.toString())
{
}

armarx::aron::error::AronException&
armarx::aron::error::AronException::operator()(const std::string& additionalMessage)
{
    auto currentReason = getReason();
    if (not additionalMessage.empty())
    {
        setReason(currentReason + ". Additional Message: " + additionalMessage);
    }
    return *this;
}

armarx::aron::error::AronEOFException::AronEOFException(const std::string& prettymethod) :
    AronException(prettymethod,
                  "REACHED THE END OF A NON VOID METHOD. PERHAPS YOU FORGOT TO ADD A VALUE "
                  "TO SOME SWITCH-CASE STATEMEMT?.")
{
}

armarx::aron::error::NotImplementedYetException::NotImplementedYetException(
    const std::string& prettymethod) :
    AronException(prettymethod, "This method is not yet implemented!")
{
}

armarx::aron::error::AronNotValidException::AronNotValidException(
    const std::string& prettymethod,
    const std::string& reason,
    const data::dto::GenericDataPtr& data) :
    AronException(prettymethod, reason + ". The ice_id of the data was: " + data->ice_id())
{
}

armarx::aron::error::AronNotValidException::AronNotValidException(
    const std::string& prettymethod,
    const std::string& reason,
    const data::dto::GenericDataPtr& data,
    const Path& path) :
    AronException(prettymethod, reason + ". The ice_id of the data was: " + data->ice_id(), path)
{
}

armarx::aron::error::AronNotValidException::AronNotValidException(
    const std::string& prettymethod,
    const std::string& reason,
    const type::dto::GenericTypePtr& type) :
    AronException(prettymethod, reason + ". The ice_id of the type was: " + type->ice_id())
{
}

armarx::aron::error::AronNotValidException::AronNotValidException(
    const std::string& prettymethod,
    const std::string& reason,
    const type::dto::GenericTypePtr& type,
    const Path& path) :
    AronException(prettymethod, reason + ". The ice_id of the type was: " + type->ice_id(), path)
{
}

armarx::aron::error::ValueNotValidException::ValueNotValidException(const std::string& prettymethod,
                                                                    const std::string& reason,
                                                                    const std::string& input) :
    AronException(prettymethod, reason + ". Got: " + input)
{
}

armarx::aron::error::ValueNotValidException::ValueNotValidException(const std::string& prettymethod,
                                                                    const std::string& reason,
                                                                    const std::string& input,
                                                                    const Path& path) :
    AronException(prettymethod, reason + ". Got: " + input, path)
{
}

armarx::aron::error::ValueNotValidException::ValueNotValidException(
    const std::string& prettymethod,
    const std::string& reason,
    const std::string& input,
    const std::string& expectation) :
    AronException(prettymethod, reason + ". Got: " + input + ". Expected: " + expectation)
{
}

armarx::aron::error::ValueNotValidException::ValueNotValidException(const std::string& prettymethod,
                                                                    const std::string& reason,
                                                                    const std::string& input,
                                                                    const std::string& expectation,
                                                                    const Path& path) :
    AronException(prettymethod, reason + ". Got: " + input + ". Expected: " + expectation, path)
{
}
