/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include "AronGeneratedClass.h"

namespace armarx::aron::cpp
{

    class AronGeneratedObjectBase : public AronGeneratedClass
    {
    public:
        AronGeneratedObjectBase() = default;
        virtual ~AronGeneratedObjectBase() = default;

        /// Convert the current bo to an aron variant data dict
        virtual armarx::aron::data::DictPtr toAron() const = 0;

        /// Convert the current bo to the ice representation of an aron variant data dict
        virtual armarx::aron::data::dto::DictPtr toAronDTO() const = 0;

        /// Set all members of the current bo according to the aron variant data dict
        virtual void fromAron(const armarx::aron::data::DictPtr& input) = 0;

        /// Set all members of the current bo according to the ice representation of an aron variant data dict
        virtual void fromAron(const armarx::aron::data::dto::DictPtr& input) = 0;
    };

    template <class Derived>
    class AronGeneratedObject : public AronGeneratedObjectBase
    {
    public:
        AronGeneratedObject() = default;
        virtual ~AronGeneratedObject() = default;

        template <class T>
        void
        to(T& t) const
        {
            const Derived* d = dynamic_cast<const Derived*>(this);
            if (d)
            {
                t.fromAron(*d);
            }
        }

        template <class T>
        void
        from(const T& t)
        {
            Derived* d = dynamic_cast<Derived*>(this);
            if (d)
            {
                t.toAron(*d);
            }
        }

        template <class T>
        void
        to(T& t, void (*fromAron)(const Derived&, T&)) const
        {
            const Derived* d = dynamic_cast<const Derived*>(this);
            if (d)
            {
                fromAron(*d, t);
            }
        }

        template <class T>
        void
        from(const T& t, void (*toAron)(Derived&, const T&))
        {
            Derived* d = dynamic_cast<Derived*>(this);
            if (d)
            {
                toAron(*d, t);
            }
        }
    };

    template <class T>
    concept isAronGeneratedObject = std::is_base_of<AronGeneratedObjectBase, T>::value;
} // namespace armarx::aron::cpp

namespace armarx::aron::codegenerator::cpp
{
    using AronGeneratedObjectBase = aron::cpp::AronGeneratedObjectBase;

    template <class Derived>
    using AronGeneratedObject = aron::cpp::AronGeneratedObject<Derived>;
} // namespace armarx::aron::codegenerator::cpp
