/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller ( fabian dot peller at kit dot edu )
 * @date       2023
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <algorithm>
#include <map>
#include <vector>

#include <SimoxUtility/algorithm/string/string_tools.h>

#include <RobotAPI/libraries/aron/core/data/variant/All.h>
#include <RobotAPI/libraries/aron/core/data/visitor/variant/VariantVisitor.h>

namespace armarx::aron::data::filter
{
    class WhitelistFilter : public aron::data::ConstVariantVisitor
    {
    public:
        struct Normalization
        {
            float min;
            float max;
        };

        WhitelistFilter(const std::vector<std::string>& whitelist,
                        const std::map<std::string, Normalization>& normalization = {});

        void visitAronVariant(const data::DictPtr&) override;
        void visitAronVariant(const data::ListPtr&) override;
        void visitAronVariant(const data::NDArrayPtr&) override;
        void visitAronVariant(const data::IntPtr&) override;
        void visitAronVariant(const data::LongPtr&) override;
        void visitAronVariant(const data::FloatPtr&) override;
        void visitAronVariant(const data::DoublePtr&) override;
        void visitAronVariant(const data::BoolPtr&) override;
        void visitAronVariant(const data::StringPtr&) override;

    private:
        bool pathInWhitelist(const aron::Path& p) const;

        // normalization
        template <class T>
        T
        checkForNormalization(const aron::Path& p, const T& t)
        {
            for (const auto& [n, v] : normalization)
            {
                if (simox::alg::starts_with(p.toString(), n))
                {
                    auto diff = v.max - v.min;
                    auto norm = (t - v.min) / diff;
                    return norm;
                }
            }
            return t;
        }

        std::vector<std::string> whitelist;
        std::map<std::string, Normalization> normalization;

    public:
        aron::data::VariantPtr data;
    };
} // namespace armarx::aron::data::filter
