#include "FloatSimilarity.h"

#include <cmath>

#include "ArmarXCore/core/logging/Logging.h"

namespace armarx::aron::similarity
{

    double
    FloatSimilarity::calculate_similarity(armarx::aron::data::FloatPtr f1,
                                          armarx::aron::data::FloatPtr f2,
                                          Type t)
    {
        switch (t)
        {
            case Type::MAE:
                return calculateMAE(f1, f2);
            case Type::MSE:
                return calculateMSE(f1, f2);
            default:
                ARMARX_INFO << "Trying to calculate similarity with unknown similarity type";
                return -1;
        }
    }

    double
    FloatSimilarity::calculateMAE(armarx::aron::data::FloatPtr f1, armarx::aron::data::FloatPtr f2)
    {

        return std::abs(f1->getValue() - f2->getValue());
    }

    double
    FloatSimilarity::calculateMSE(armarx::aron::data::FloatPtr f1, armarx::aron::data::FloatPtr f2)
    {
        return std::pow(f1->getValue() - f2->getValue(), 2);
    }


} // namespace armarx::aron::similarity
