#include "NDArraySimilarity.h"

#include <cmath>

#include <ArmarXCore/core/logging/Logging.h>

#include "../../cosine.h"
#include "chernoff.h"
#include "mae.h"
#include "mse.h"

namespace armarx::aron::similarity
{


    double
    NDArraySimilarity::calculate_similarity(data::NDArrayPtr p1, data::NDArrayPtr p2, Type type)
    {
        switch (type)
        {
            case Type::MSE:
                //ARMARX_INFO << "Calculate MSE";
                return armarx::aron::similarity::mse::compute_similarity(p1, p2);
            case Type::MAE:
                return armarx::aron::similarity::mae::compute_similarity(p1, p2);
            case Type::CHERNOFF:
                return armarx::aron::similarity::chernoff::compute_similarity(p1, p2);
            case Type::COSINE:
                return armarx::aron::similarity::cosine::compute_similarity(p1, p2);
            default:
                ARMARX_WARNING
                    << "Trying to calculate similarity with unspecified similarity measurement";
                return -1;
        }
    }

    std::string
    NDArraySimilarity::to_string(Type t)
    {
        switch (t)
        {
            case MSE:
                return "MSE";
            case MAE:
                return "MAE";
            case CHERNOFF:
                return "Chernoff";
            case COSINE:
                return "Cosine";
            default:
                return "No similarity type information";
        }
    }

    double
    NDArraySimilarity::calculate_similarity_multi(std::vector<data::NDArrayPtr> images,
                                                  armarx::aron::data::NDArrayPtr p,
                                                  Type type)
    {
        double sim = 0;
        for (auto& image : images)
        {
            //ARMARX_INFO << "Before calculation";
            sim += calculate_similarity(image, p, type);
            //ARMARX_INFO << "Sim is currently: " << sim;
        }
        return sim /
               (images.size() +
                1); // to average it over the distances, makes it easier to find good parameters
    }

} // namespace armarx::aron::similarity
