#include "mse.h"

#include <cmath>

#include <SimoxUtility/algorithm/string.h>

#include <ArmarXCore/core/logging/Logging.h>

namespace armarx::aron::similarity
{
    double
    mse::compute_similarity(const aron::data::NDArrayPtr p1, const aron::data::NDArrayPtr p2)
    {
        //ARMARX_INFO << "Begin MSE";
        //auto start = std::chrono::high_resolution_clock::now();
        double sum = 0;

        //TODO: check shapes have same size
        //ARMARX_INFO << "One";
        if (p1 != nullptr && p2 != nullptr)
        {
            p1->getShortName();
        }
        else
        {
            if (p1 == nullptr)
            {
                ARMARX_INFO << "P1 is nullptr";
            }
            else
            {
                ARMARX_INFO << "P2 is Nullpointer";
            }
        }
        //ARMARX_INFO << "Two";

        //return 0;

        std::vector<unsigned char> first_image = p1->getDataAsVector();
        std::vector<unsigned char> second_image = p2->getDataAsVector();
        int size = first_image.size();

        //ARMARX_INFO << "Image size: " << std::to_string(int(first_image.size()));
        //ARMARX_INFO << "Image size: " << std::to_string(int(second_image.size()));
        ARMARX_CHECK(first_image.size() == second_image.size());

        //auto start = std::chrono::high_resolution_clock::now();
        int rolling_number = 1; //TODO: make sure that no elements will be left over

        for (int i = 0; i < int(first_image.size()); i += rolling_number)
        {
            //loop unrolling to shorten the needed computation time:
            sum += std::pow(first_image.at(i) - second_image.at(i), 2);
            /*
            sum += std::pow(first_image.at(i + 1) - second_image.at(i + 1), 2);
            sum += std::pow(first_image.at(i + 2) - second_image.at(i + 2), 2);
            sum += std::pow(first_image.at(i + 3) - second_image.at(i + 3), 2);
            */
        }


        //auto end = std::chrono::high_resolution_clock::now();
        //auto additional = std::chrono::duration_cast<std::chrono::microseconds>(end - start);
        //ARMARX_INFO << "Time with unroll factor " << rolling_number << " MSE: " << std::to_string(additional.count());
        first_image.clear();
        second_image.clear();

        //ARMARX_INFO << "MSE is: " << std::to_string(sum / size);

        return sum / (size);
    }

} // namespace armarx::aron::similarity
