/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    robdekon
 * @author     Christoph Pohl ( christoph dot pohl at kit dot edu )
 * @date       07.09.22
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <experimental/memory>

#include <ArmarXCore/core/application/properties/forward_declarations.h>

#include <RobotAPI/libraries/aron/core/data/visitor/variant/VariantVisitor.h>

namespace armarx::RemoteGui
{
    namespace detail
    {
        class GroupBoxBuilder;
    }
    class TabProxy;

    class ValueVariant;

    using ValueMap = std::map<std::string, ValueVariant>;
} // namespace armarx::RemoteGui

namespace armarx::aron::component_config
{

    class MakeConfigGuiVisitor : public data::RecursiveConstTypedVariantVisitor
    {
    public:
        explicit MakeConfigGuiVisitor(const std::string& name);

        void visitObjectOnEnter(DataInput&, TypeInput&) override;

        void visitObjectOnExit(DataInput&, TypeInput&) override;

        void visitListOnEnter(DataInput&, TypeInput&) override;

        void visitListOnExit(DataInput&, TypeInput&) override;

        void visitDictOnEnter(DataInput&, TypeInput&) override;

        void visitDictOnExit(DataInput&, TypeInput&) override;

        void visitInt(DataInput&, TypeInput&) override;

        void visitIntEnum(DataInput&, TypeInput&) override;

        void visitFloat(DataInput&, TypeInput&) override;

        void visitDouble(DataInput&, TypeInput&) override;

        void visitBool(DataInput&, TypeInput&) override;

        void visitString(DataInput&, TypeInput&) override;

        void visitQuaternion(DataInput& input, TypeInput& typeInput) override;

        void visitMatrix(DataInput& input, TypeInput& typeInput) override;

        [[nodiscard]] RemoteGui::detail::GroupBoxBuilder getGroupBoxBuilder() const;

    private:
        std::shared_ptr<RemoteGui::detail::GroupBoxBuilder> builder_;
        std::vector<std::shared_ptr<RemoteGui::detail::GroupBoxBuilder>> group_hierarchy_;
        std::atomic_bool in_list_{false};
    };

    class GetValueFromMapVisitor :
        public aron::data::RecursiveTypedVisitor<data::VariantPtr, const type::VariantPtr>
    {
    public:
        explicit GetValueFromMapVisitor(armarx::RemoteGui::TabProxy* proxy);

        type::Descriptor getDescriptor(DataInput&, TypeInput&) override;

        MapElements getObjectElements(DataInput&, TypeInput&) override;

        MapElements getDictElements(DataInput&, TypeInput&) override;

        ListElements getListElements(DataInput&, TypeInput&) override;

        PairElements getPairElements(DataInput&, TypeInput&) override;

        TupleElements getTupleElements(DataInput&, TypeInput&) override;

        void visitListOnEnter(DataInput&, TypeInput&) override;

        void visitListOnExit(DataInput&, TypeInput&) override;

        void visitDictOnEnter(DataInput&, TypeInput&) override;

        void visitDictOnExit(DataInput&, TypeInput&) override;

        void visitInt(DataInput&, TypeInput&) override;

        void visitIntEnum(DataInput&, TypeInput&) override;

        void visitFloat(DataInput&, TypeInput&) override;

        void visitDouble(DataInput&, TypeInput&) override;

        void visitBool(DataInput&, TypeInput&) override;

        void visitString(DataInput&, TypeInput&) override;
        void visitMatrix(DataInput& elementData, TypeInput& elementType) override;
        void visitQuaternion(DataInput& elementData, TypeInput& elementType) override;

        bool tabRebuildRequired() const;

    private:
        std::experimental::observer_ptr<armarx::RemoteGui::TabProxy> proxy_;
        bool in_list_{false};
        bool tab_rebuild_required_{false};
    };

} // namespace armarx::aron::component_config