/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI
 * @author     Christoph Pohl ( christoph dot pohl at kit dot edu )
 * @date       04.11.22
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <list>

#include <ArmarXCore/core/application/properties/forward_declarations.h>
#include <ArmarXCore/util/CPPUtility/SelfRegisteringFactory.h>

#include <RobotAPI/libraries/aron/core/Descriptor.h>
#include <RobotAPI/libraries/aron/core/data/variant/Variant.h>

namespace armarx::aron::component_config
{
    namespace factories
    {
        struct VariantHelper : armarx::Factory<VariantHelper, type::Descriptor>
        {
            explicit VariantHelper(Key){};

            virtual ~VariantHelper() = default;

            virtual std::string to_string(const armarx::aron::data::VariantPtr&) const = 0;

            virtual void set_value_from_string(const armarx::aron::data::VariantPtr&,
                                               const std::string& string) const = 0;

            virtual aron::data::VariantPtr from_string(const std::string&,
                                                       const armarx::aron::Path& path) const = 0;
        };
    } // namespace factories

    namespace products
    {
        template <type::Descriptor DescT>
        struct VariantHelper : factories::VariantHelper::Registrar<VariantHelper<DescT>>
        {
            using RegistrarT = factories::VariantHelper::Registrar<VariantHelper<DescT>>;
            static constexpr type::Descriptor id = DescT;

            explicit VariantHelper() : RegistrarT(typename RegistrarT::Registration()){};

            [[nodiscard]] std::string to_string(const data::VariantPtr& ptr) const override;

            void set_value_from_string(const armarx::aron::data::VariantPtr& variant,
                                       const std::string& string) const override;

            [[nodiscard]] aron::data::VariantPtr
            from_string(const std::string&, const armarx::aron::Path& path) const override;
        };

    } // namespace products

    template <>
    std::string
    products::VariantHelper<type::Descriptor::INT>::to_string(const data::VariantPtr& ptr) const;

    template <>
    aron::data::VariantPtr products::VariantHelper<type::Descriptor::INT>::from_string(
        const std::string&,
        const armarx::aron::Path& path) const;

    template <>
    void products::VariantHelper<type::Descriptor::INT>::set_value_from_string(
        const armarx::aron::data::VariantPtr&,
        const std::string&) const;

    template <>
    std::string
    products::VariantHelper<type::Descriptor::FLOAT>::to_string(const data::VariantPtr& ptr) const;

    template <>
    aron::data::VariantPtr products::VariantHelper<type::Descriptor::FLOAT>::from_string(
        const std::string&,
        const armarx::aron::Path& path) const;

    template <>
    void products::VariantHelper<type::Descriptor::FLOAT>::set_value_from_string(
        const armarx::aron::data::VariantPtr&,
        const std::string&) const;

    template <>
    std::string
    products::VariantHelper<type::Descriptor::BOOL>::to_string(const data::VariantPtr& ptr) const;

    template <>
    aron::data::VariantPtr products::VariantHelper<type::Descriptor::BOOL>::from_string(
        const std::string&,
        const armarx::aron::Path& path) const;

    template <>
    void products::VariantHelper<type::Descriptor::BOOL>::set_value_from_string(
        const armarx::aron::data::VariantPtr&,
        const std::string&) const;

    template <>
    std::string
    products::VariantHelper<type::Descriptor::STRING>::to_string(const data::VariantPtr& ptr) const;

    template <>
    aron::data::VariantPtr products::VariantHelper<type::Descriptor::STRING>::from_string(
        const std::string&,
        const armarx::aron::Path& path) const;

    template <>
    void products::VariantHelper<type::Descriptor::STRING>::set_value_from_string(
        const armarx::aron::data::VariantPtr&,
        const std::string&) const;

    template <>
    std::string
    products::VariantHelper<type::Descriptor::DOUBLE>::to_string(const data::VariantPtr& ptr) const;

    template <>
    aron::data::VariantPtr products::VariantHelper<type::Descriptor::DOUBLE>::from_string(
        const std::string&,
        const armarx::aron::Path& path) const;

    template <>
    void products::VariantHelper<type::Descriptor::DOUBLE>::set_value_from_string(
        const armarx::aron::data::VariantPtr&,
        const std::string&) const;

    template <>
    std::string products::VariantHelper<type::Descriptor::INT_ENUM>::to_string(
        const data::VariantPtr& ptr) const;

    template <>
    aron::data::VariantPtr products::VariantHelper<type::Descriptor::INT_ENUM>::from_string(
        const std::string&,
        const armarx::aron::Path& path) const;

    template <>
    void products::VariantHelper<type::Descriptor::INT_ENUM>::set_value_from_string(
        const armarx::aron::data::VariantPtr&,
        const std::string&) const;

    static inline const std::list<type::Descriptor> implementedListDescriptors = {
        type::Descriptor::INT_ENUM,
        type::Descriptor::INT,
        type::Descriptor::STRING,
        type::Descriptor::FLOAT,
        type::Descriptor::DOUBLE,
        type::Descriptor::BOOL,
    };
    extern template struct products::VariantHelper<type::Descriptor::INT>;
    extern template struct products::VariantHelper<type::Descriptor::FLOAT>;
    extern template struct products::VariantHelper<type::Descriptor::DOUBLE>;
    extern template struct products::VariantHelper<type::Descriptor::STRING>;
    extern template struct products::VariantHelper<type::Descriptor::BOOL>;
    extern template struct products::VariantHelper<type::Descriptor::INT_ENUM>;
} // namespace armarx::aron::component_config
