/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI
 * @author     Christoph Pohl ( christoph dot pohl at kit dot edu )
 * @date       17.02.23
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <VirtualRobot/VirtualRobot.h>

#include <RobotAPI/components/ArViz/Client/Client.h>
#include <RobotAPI/libraries/ArmarXObjects/ObjectPoseClient.h>
#include <RobotAPI/libraries/ArmarXObjects/forward_declarations.h>
#include <RobotAPI/libraries/armem_vision/OccupancyGridHelper.h>
#include <RobotAPI/libraries/armem_vision/types.h>

namespace armarx::obstacle_avoidance
{

    class CollisionModelHelper
    {
    public:
        using ManipulationObjectSet = std::vector<VirtualRobot::ManipulationObject>;
        using ManipulationObjectSetPtr = std::shared_ptr<ManipulationObjectSet>;

        static VirtualRobot::ManipulationObjectPtr
        asManipulationObject(const objpose::ObjectPose& objectPose);
        static ManipulationObjectSetPtr
        asManipulationObjects(const objpose::ObjectPoseSeq& objectPoses);
        static VirtualRobot::SceneObjectSetPtr
        asSceneObjects(const objpose::ObjectPoseSeq& objectPoses);
        static VirtualRobot::SceneObjectSetPtr
        asSceneObjects(const armem::vision::OccupancyGrid& occupancyGrid,
                       const OccupancyGridHelper::Params& params);
        static void visualizeCollisionModel(const VirtualRobot::CollisionModelPtr& model,
                                            viz::Client& arviz);


        CollisionModelHelper(const objpose::ObjectPoseClient& client);
        VirtualRobot::SceneObjectSetPtr fetchSceneObjects();
        ManipulationObjectSetPtr fetchManipulationObjects();

    private:
        objpose::ObjectPoseClient objectPoseClient_;
    };
} // namespace armarx::obstacle_avoidance
