#include "SkillExecutionID.h"

#include <ArmarXCore/core/time/ice_conversions.h>

namespace armarx
{
    namespace skills
    {

        skills::manager::dto::SkillExecutionID
        SkillExecutionID::toManagerIce() const
        {
            skills::manager::dto::SkillExecutionID ret;
            ret.skillId = skillId.toManagerIce();
            ret.executorName = executorName;
            armarx::core::time::toIce(ret.executionStartedTime, executionStartedTime);
            return ret;
        }

        skills::provider::dto::SkillExecutionID
        SkillExecutionID::toProviderIce() const
        {
            skills::provider::dto::SkillExecutionID ret;
            ret.skillId = skillId.toProviderIce();
            ret.executorName = executorName;
            armarx::core::time::toIce(ret.executionStartedTime, executionStartedTime);
            return ret;
        }

        SkillExecutionID
        SkillExecutionID::FromIce(const skills::manager::dto::SkillExecutionID& i)
        {
            armarx::core::time::DateTime t;
            armarx::core::time::fromIce(i.executionStartedTime, t);
            return {.skillId = skills::SkillID::FromIce(i.skillId),
                    .executorName = i.executorName,
                    .executionStartedTime = t};
        }

        SkillExecutionID
        SkillExecutionID::FromIce(const skills::provider::dto::SkillExecutionID& i,
                                  const std::optional<skills::ProviderID>& providerName)
        {
            armarx::core::time::DateTime t;
            armarx::core::time::fromIce(i.executionStartedTime, t);
            return {skills::SkillID::FromIce(i.skillId, providerName), i.executorName, t};
        }

        std::string
        SkillExecutionID::toString() const
        {
            return skillId.toString() + ENTER_SEPARATOR + executorName + SEPARATOR +
                   executionStartedTime.toDateTimeString() + EXIT_SEPARATOR;
        }

        bool
        SkillExecutionID::operator<=(const SkillExecutionID& other) const
        {
            return this->executionStartedTime <= other.executionStartedTime;
        }

        bool
        SkillExecutionID::operator<(const SkillExecutionID& other) const
        {
            return this->executionStartedTime < other.executionStartedTime;
        }

        bool
        SkillExecutionID::operator==(const SkillExecutionID& other) const
        {
            return this->toString() == other.toString();
        }
    } // namespace skills
} // namespace armarx
