#pragma once

#include <ArmarXCore/core/logging/Logging.h>

#include <RobotAPI/libraries/skills/core/SkillDescription.h>
#include <RobotAPI/libraries/skills/core/SkillStatusUpdate.h>

namespace armarx
{
    namespace skills
    {
        /* Manages the remote execution of a skill and converts the ice types */
        class SkillProxy : public armarx::Logging
        {
        public:
            /// We remove the default constructor as every skill proxy requires a manager
            SkillProxy() = delete;

            /// set the skill proxy using a skillId. Queries the manager to get the description.
            SkillProxy(const manager::dti::SkillManagerInterfacePrx& manager,
                       const SkillID& skillId);

            /// set the proxy using a skill description
            SkillProxy(const manager::dti::SkillManagerInterfacePrx& manager,
                       const SkillDescription& skillDesc);

            /// copy ctor
            SkillProxy(const SkillProxy& o) = default;

            /// get the skill description
            SkillDescription getSkillDescription() const;

            /// get the skill id from the skill description
            SkillID getSkillId() const;

            // Provide a similar API as the skillprovider
            /// execute a skill and block until skill terminates
            TerminatedSkillStatusUpdate
            executeSkill(const std::string& executorName,
                         const aron::data::DictPtr& params = nullptr) const;

            /// execute a skill. Do not block during execution
            SkillExecutionID executeSkillAsync(const std::string& executorName,
                                               const aron::data::DictPtr& params = nullptr) const;

            /// poll execution status and block until its null or terminated
            std::optional<TerminatedSkillStatusUpdate>
            join(const SkillExecutionID& executionId) const;

            /// ask skill to abort ASAP. Blocks until skill stopped
            bool abortSkill(const SkillExecutionID& executionId) const;

            /// ask skill to abort ASAP
            bool abortSkillAsync(const SkillExecutionID& executionId) const;

            // Utiliy methods
            /// get the default parameters of the skill. TODO: Skill profiles in memory!
            aron::data::DictPtr getRootProfileParameters() const;

        protected:
            manager::dti::SkillManagerInterfacePrx manager;
            SkillDescription skillDescription;
        };
    } // namespace skills
} // namespace armarx
