#ifndef SKILLMEMORYGUI_H
#define SKILLMEMORYGUI_H

#include <QHBoxLayout>
#include <QLabel>
#include <QSplitter>
#include <QTreeWidget>
#include <QWidget>

#include <ArmarXCore/core/logging/Logging.h>

#include <ArmarXGui/libraries/ArmarXGuiBase/widgets/PeriodicUpdateWidget.h>
#include <ArmarXGui/libraries/ArmarXGuiBase/widgets/StatusLabel.h>

#include "./aron_tree_widget/widgets/SkillDescriptionWidget.h"
#include "./executions/SkillExecutionTreeWidget.h"
#include "./memory/SkillManagerWrapper.h"
#include "./skill_details/SkillDetailsGroupBox.h"
#include "./skills/SkillGroupBox.h"

namespace armarx::skills::gui
{
    class SkillMemoryGUI : public QObject, public armarx::Logging
    {
        Q_OBJECT

    public:
        static const constexpr char* STOP_ALL_BUTTON_TEXT = "Stop all executions";
        SkillMemoryGUI(QTreeWidget* _skillExecutionTreeWidget,
                       QLayout* _skillExecutionTreeWidgetParentLayout,
                       QGroupBox* _skillGroupBox,
                       QSplitter* _skillGroupBoxParentLayout,
                       QGroupBox* _skillDetailGroupBox,
                       QSplitter* _skillDetailGroupBoxParentLayout,
                       QHBoxLayout* _updateWidgetLayout,

                       QLayout* stopAllLayout,

                       QWidget* connectionStatusLabel,

                       std::shared_ptr<SkillManagerWrapper> _memory);

    signals:
        /**
         * @brief Notify affected widgets to clear all saved state.
         */
        void disconnectGui();

        /**
         * @brief Notify widgets of new connection.
         * @param observerName the observer name.
         */
        void connectGui(std::string observerName);

        void updateGui(SkillManagerWrapper::Snapshot update);

    private:
        void setupUi();
        void connectSignals();

        SkillExecutionTreeWidget* skillExecutionTreeWidget = nullptr;
        SkillGroupBox* skillGroupBox = nullptr;
        SkillDetailGroupBox* skillDetailGroupBox = nullptr;
        QBoxLayout* updateWidgetLayout = nullptr;

        std::shared_ptr<SkillManagerWrapper> memory = nullptr;

        PeriodicUpdateWidget* updateWidget = nullptr;

        QPushButton* stopAllButton = nullptr;

        StatusLabel* connectionStatusLabel = nullptr;
    };
} // namespace armarx::skills::gui

#endif // SKILLMEMORYGUI_H
