#pragma once

#include <QTreeWidget>

#include <ArmarXCore/core/system/ImportExportComponent.h>

#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXComponentWidgetController.h>
#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXGuiPlugin.h>

#include <RobotAPI/libraries/aron/core/type/variant/Variant.h>

#include "Data.h"

namespace armarx::skills::gui
{
    // Internal derived QtTreeWidgetItem class. Contains additional information to compare its data agains aron types.
    // It contains 3 columns: Key ; Value ; Type
    // The key can only be edited if the parent object is a dict
    class AronTreeWidgetItem : public QObject, public QTreeWidgetItem
    {
        Q_OBJECT
    public:
        AronTreeWidgetItem(bool editKey, bool editVal, QString key, aron::type::VariantPtr type);

        using QTreeWidgetItem::QTreeWidgetItem;

        static AronTreeWidgetItem* DynamicCast(QTreeWidgetItem*);
        static AronTreeWidgetItem* DynamicCastAndCheck(QTreeWidgetItem*);

        aron::type::VariantPtr aronType;
        // if editing the first column should be allowed
        const bool col0Editable = false;
        // if editing the second column should be allowed
        const bool col1Editable = false;

        bool isValueErrorneous();
        // marks the gui in a specific color. Also stores if there is currently an error.
        // the only way to reset the color back to normal is with resetError() (error is sticky)
        void setValueErrorState(bool isErrorSource, bool isTransitiveError);
        void setKeyErrorState(bool hasError);
        // reset error storage that influences coloring for this and all children
        void resetError();

        // Checks if the children of a dict are unique
        // should not be called on other types! (does nothing then)
        void checkKeyValidityOfChildren();

        // main logic on changes. Gets called from the onTreeWidgetItemChanged() slot in AronTreeWidgetController.
        // (This class cannot directly consume this signal, at least I did not find a nice way...)
        void onUserChange(int changedColumn);

    private:
        // because the editable keyword counts for all columns, it is possible to TAB into uneditable keys.
        // We do not want those to change, so just change them back once they finished editing.
        void preventIllegalKeyChange();

        bool itemValueError = false;
        bool transitiveValueError = false;
        bool keyValueError = false;
        // hacky storage of previous key value for items that usually should not be editable.
        // Problem: Dict-keys are editable, one can tab from that to any other value field and edit it.
        // Fix: Once the change is commited, the AronTreeWidgetController notices the change and checks if it was allowed.
        QString unchangeableKey = "";
    };
} // namespace armarx::skills::gui
