/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::gui-plugins::SkillManagerMonitorWidgetController
 * @author     Raphael Grimm ( raphael dot grimm at kit dot edu )
 * @date       2020
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include <QTreeWidgetItem>

#include <RobotAPI/libraries/aron/core/data/variant/All.h>
#include <RobotAPI/libraries/aron/core/type/variant/All.h>
#include <RobotAPI/libraries/aron/core/type/visitor/variant/VariantVisitor.h>

namespace armarx::skills::gui
{
    class AronTreeWidgetItem;

    // Convert aron type to tree widgets
    class AronTreeWidgetCreatorVisitor : public armarx::aron::type::ConstVariantVisitor
    {
    public:
        AronTreeWidgetItem* createdQWidgetItem = nullptr;

        AronTreeWidgetCreatorVisitor() = delete;

        // Takes the parent tree element and attaches the newly crated object during visit() to it.
        // This allows us to also use information of the parent object during creation (like if this child is a dict entry or part of a list)
        // IMPORTANT: For the root element, manually set the topLevelWidget after this constructor
        AronTreeWidgetCreatorVisitor(QTreeWidgetItem* parentInstance);

        void insertNewTreeViewWidget(Input& i, const std::string&);


        void visitAronVariant(const aron::type::ObjectPtr&) final;
        void visitAronVariant(const aron::type::DictPtr& i) final;
        void visitAronVariant(const aron::type::PairPtr& i) final;
        void visitAronVariant(const aron::type::TuplePtr& i) final;
        void visitAronVariant(const aron::type::ListPtr& i) final;
        void visitAronVariant(const aron::type::NDArrayPtr& i) final;
        void visitAronVariant(const aron::type::MatrixPtr& i) final;
        void visitAronVariant(const aron::type::QuaternionPtr& i) final;
        void visitAronVariant(const aron::type::ImagePtr& i) final;
        void visitAronVariant(const aron::type::PointCloudPtr& i) final;
        void visitAronVariant(const aron::type::IntEnumPtr& i) final;
        void visitAronVariant(const aron::type::IntPtr& i) final;
        void visitAronVariant(const aron::type::LongPtr& i) final;
        void visitAronVariant(const aron::type::FloatPtr& i) final;
        void visitAronVariant(const aron::type::DoublePtr& i) final;
        void visitAronVariant(const aron::type::BoolPtr& i) final;
        void visitAronVariant(const aron::type::StringPtr& i) final;
        void visitUnknown(Input&) final;

        // setter for the widget attachment point. Only needs to be manually set for the root element.
        void setTopLevelWidget(QTreeWidget* widget);

    private:
        void handleEditable();
        std::string generateUniqueKeyFromSet(std::set<std::string>&& usedKeys);
        QTreeWidgetItem* parentOfCreatedObj = nullptr;
        QTreeWidget* toplevelWidget = nullptr;
        const std::string defaultMapKeyName = "key_";
        // controls, if values (column 1) can be edited directly
        bool editableValue = false;
    };
} // namespace armarx::skills::gui
