#include "SkillExecutionTreeWidgetItem.h"

namespace armarx::skills::gui
{
    SkillExecutionTreeWidgetItem*
    SkillExecutionTreeWidgetItem::SearchRecursiveForMatch(SkillExecutionTreeWidgetItem* haystack,
                                                          const SkillExecutionID& needle)
    {
        if (!haystack)
        {
            return nullptr;
        }

        if (needle == haystack->executionId)
        {
            return haystack;
        }
        for (int i = 0; i < haystack->childCount(); ++i)
        {
            auto el = dynamic_cast<SkillExecutionTreeWidgetItem*>(haystack->child(i));
            if (!el)
            {
                // Dynamic casting failed. Skip element...
                // We assume, that only this type can contain children in the tree.
                continue;
            }
            return SkillExecutionTreeWidgetItem::SearchRecursiveForMatch(el, needle);
        }
        return nullptr;
    }

    SkillExecutionID
    SkillExecutionTreeWidgetItem::getExecutionId()
    {
        return this->executionId;
    }

    void
    SkillExecutionTreeWidgetItem::updateItem(skills::SkillStatus& skillStatus)
    {
        this->setText(0,
                      QString::fromStdString(executionId.executionStartedTime.toDateTimeString()));
        this->setText(1, QString::fromStdString(executionId.executorName));
        this->setText(2, QString::fromStdString(executionId.skillId.toString()));
        for (std::pair<skills::SkillStatus, std::string> i : EXECUTION_STATUS_TO_STRING)
        {
            if (i.first == skillStatus)
            {
                this->setText(3, QString::fromStdString(i.second));
            }
        }
        updateButtonState(skillStatus);
    }

    void
    SkillExecutionTreeWidgetItem::abortExecution()
    {
        memory->stopExecution(executionId);
    }

    void
    SkillExecutionTreeWidgetItem::setupUi()
    {
        parentTree->insertTopLevelItem(0, this);
        abortButton = new QPushButton();
        abortButton->setMinimumHeight(20);
        abortButton->setMaximumHeight(20);
        parentTree->setItemWidget(this, 4, abortButton);

        QPixmap pix(":/icons/process-stop-7.ico");
        QIcon icon(pix);

        abortButton->setIcon(icon);


        connectSignals();
    }

    void
    SkillExecutionTreeWidgetItem::connectSignals()
    {
        connect(abortButton,
                &QPushButton::clicked,
                this,
                &SkillExecutionTreeWidgetItem::abortExecution);
    }

    void
    SkillExecutionTreeWidgetItem::updateButtonState(SkillStatus& skillStatus)
    {
        abortButton->setDisabled(skillStatus == SkillStatus::Aborted ||
                                 skillStatus == SkillStatus::Failed ||
                                 skillStatus == SkillStatus::Succeeded);
    }
} // namespace armarx::skills::gui
