#ifndef SKILLEXECUTIONTREEWIDGETITEM_H
#define SKILLEXECUTIONTREEWIDGETITEM_H

#include <QPushButton>
#include <QTreeWidgetItem>

#include "RobotAPI/libraries/skills_gui/executions/SkillExecutionTreeWidget.h"
#include <RobotAPI/libraries/skills/core/SkillExecutionID.h>
#include <RobotAPI/libraries/skills/core/SkillStatusUpdate.h>

#include "../memory/MemoryCommunicatorBase.h"
#include "SkillExecutionTreeWidgetItem.h"

namespace armarx::skills::gui
{
    static const std::map<skills::SkillStatus, std::string> EXECUTION_STATUS_TO_STRING = {
        // Main states
        {skills::SkillStatus::Constructing, "Constructing"},
        {skills::SkillStatus::Initializing, "Initializing"},
        {skills::SkillStatus::Preparing, "Preparing"},
        {skills::SkillStatus::Running, "Running"},

        // Terminating
        {skills::SkillStatus::Aborted, "Aborted"},
        {skills::SkillStatus::Failed, "Failed"},
        {skills::SkillStatus::Succeeded, "Succeeded"}};

    class SkillExecutionTreeWidgetItem :
        public QObject,
        public QTreeWidgetItem,
        public MemoryCommunicatorBase
    {
        Q_OBJECT
    public:
        SkillExecutionTreeWidgetItem() = delete;

        SkillExecutionTreeWidgetItem(const skills::SkillExecutionID& id,
                                     std::shared_ptr<SkillManagerWrapper> _memory,
                                     SkillExecutionTreeWidget* parent) :
            MemoryCommunicatorBase(_memory), executionId(id), parentTree(parent)
        {
            setupUi();
        }

        static SkillExecutionTreeWidgetItem*
        SearchRecursiveForMatch(SkillExecutionTreeWidgetItem* haystack,
                                const skills::SkillExecutionID& needle);

        skills::SkillExecutionID getExecutionId();

        void updateItem(skills::SkillStatus& skillStatus);

    protected:
        skills::SkillExecutionID executionId;

    public slots:
        void abortExecution();

    private:
        void setupUi();
        void connectSignals();
        void updateButtonState(skills::SkillStatus& skillStatus);
        SkillExecutionTreeWidget* parentTree = nullptr;
        QPushButton* abortButton = nullptr;
    };
} // namespace armarx::skills::gui

#endif // SKILLEXECUTIONTREEWIDGETITEM_H
