#ifndef SKILLGROUPBOX_H
#define SKILLGROUPBOX_H

#include <QGroupBox>
#include <QLineEdit>
#include <QPushButton>

#include "../memory/MemoryCommunicatorBase.h"
#include "SkillTreeWidget.h"

namespace armarx::skills::gui
{
    class SkillGroupBox : public QGroupBox, public MemoryCommunicatorBase
    {
        Q_OBJECT
    public:
        static const constexpr char* GROUP_BOX_TITLE = "Skills";

        SkillGroupBox(std::shared_ptr<SkillManagerWrapper> _memory, QWidget* parent = nullptr) :
            QGroupBox(parent), MemoryCommunicatorBase(_memory)
        {
            setupUi();
        }

    signals:
        /**
         * @brief Output signal to controller.
         */
        void searchRequest(std::string search);

        /**
         * @brief Notify the skill description widget of an updated selection.
         * @param skillId
         */
        void updateSkillDetails(skills::SkillID& skillId);

        void disconnectGui();

    public slots:
        void connectGui(std::string observerName);

        void updateGui(SkillManagerWrapper::Snapshot update);

        // is called, after the search has been accepted in the memory wrapper.
        void searchCallback();

    private slots:
        /**
         * Takes string from line edit and emits the search request.
         */
        void handleSearch();

        /**
         * @brief Applies the update, and requests an update from memory.
         */
        void filterAndFetch();

    private:
        void setupUi();
        void connectSignals();
        QLineEdit* searchBar = nullptr;
        QPushButton* acceptSearchButton = nullptr;
        SkillTreeWidget* skillTreeWidget = nullptr;
    };
} // namespace armarx::skills::gui

#endif // SKILLGROUPBOX_H
