#include "SkillTreeWidget.h"

#include "RobotAPI/libraries/skills_gui/skills/SkillTreeWidgetItem.h"

namespace armarx::skills::gui
{
    void
    SkillTreeWidget::skillSelectionChanged(QTreeWidgetItem* current, int)
    {
        if (!current)
        {
            // gui has died?
            return;
        }

        auto c = dynamic_cast<SkillTreeWidgetItem*>(current);
        if (!c)
        {
            // we don't know what to do with the item. Abort...
            return;
        }

        selectedSkill.skillId = c->skillId;
        emit updateSkillDetails(selectedSkill.skillId);
    }

    void
    SkillTreeWidget::updateGui(SkillMap update)
    {
        setSortingEnabled(false);

        // update tree view. Remove non-existing elements
        int i = 0;
        while (i < this->topLevelItemCount())
        {
            auto* providerItem = this->topLevelItem(i);
            auto providerName = providerItem->text(0).toStdString();
            skills::ProviderID providerId{.providerName = providerName};

            if (update.find(providerId) == update.end())
            {
                providerItem = nullptr; // reset
                auto remove = this->takeTopLevelItem(i);
                delete remove;
                continue;
            }

            ++i;

            // sanity check
            ARMARX_CHECK(update.count(providerId) > 0);
            auto& providedSkills = update.at(providerId);

            int j = 0;
            while (j < providerItem->childCount())
            {
                auto* skillItem = providerItem->child(j);
                auto skillName = skillItem->text(0).toStdString();

                skills::SkillID skillId{.providerId =
                                            skills::ProviderID{.providerName = providerName},
                                        .skillName = skillName};

                if (providedSkills.find(skillId) == providedSkills.end())
                {
                    skillItem = nullptr;
                    auto remove = providerItem->takeChild(j);
                    delete remove;
                    continue;
                }

                ++j;
            }
        }

        // update tree view. Add new elements
        for (const auto& [providerId, providedSkills] : update)
        {
            bool newProvider = false;
            QTreeWidgetItem* providerItem = nullptr;
            for (int i = 0; i < this->topLevelItemCount(); ++i)
            {
                auto el = this->topLevelItem(i);
                auto providerName = el->text(0).toStdString();
                skills::ProviderID elProviderId{.providerName = providerName};

                if (providerId == elProviderId)
                {
                    providerItem = el;
                    break;
                }
            }

            if (!providerItem)
            {
                providerItem = new QTreeWidgetItem(this);
                providerItem->setText(0, QString::fromStdString(providerId.providerName));
                // expand new items
                newProvider = true;
            }

            for (const auto& [skillId, skill] : providedSkills)
            {
                QTreeWidgetItem* skillItem = nullptr;
                for (int i = 0; i < providerItem->childCount(); ++i)
                {
                    auto el = providerItem->child(i);
                    auto skillName = el->text(0).toStdString();
                    skills::SkillID elSkillId{providerId, skillName};

                    if (skillId == elSkillId)
                    {
                        skillItem = el;
                        break;
                    }
                }

                if (!skillItem)
                {
                    skillItem = new SkillTreeWidgetItem(skillId, providerItem);
                    skillItem->setText(0, QString::fromStdString(skillId.skillName));
                }
            }

            if (newProvider && PROVIDER_EXPANDED_DEFAULT)
            {
                providerItem->setExpanded(true);
            }
        }

        setSortingEnabled(true);
        sortByColumn(0, Qt::AscendingOrder);
        sortByColumn(1, Qt::AscendingOrder);
    }

    void
    SkillTreeWidget::disconnectGui()
    {
        this->clear();
        selectedSkill = SelectedSkill();
    }

    void
    SkillTreeWidget::setupUi()
    {
        this->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

        QTreeWidgetItem* qtreewidgetitem = this->headerItem();

        qtreewidgetitem->setText(0, "Skill");

        connectSignals();
    }

    void
    SkillTreeWidget::connectSignals()
    {
        // The selectionChanged signal does not trigger when clicking the active item
        // => we use itemClicked
        connect(this, &QTreeWidget::itemClicked, this, &SkillTreeWidget::skillSelectionChanged);
    }


} // namespace armarx::skills::gui
